using System;
using System.Security.Authentication;
using System.Security.Cryptography.X509Certificates;
using Autofac;
using FitConnect;
using FluentAssertions;
using NUnit.Framework;

namespace FluentApiTest;

public class FluentSubscriberPollingTests {
    private Client _client = null!;
    private string clientId = "clientId";
    private string clientSecret = "clientSecret";
    private IContainer _container = null!;
    private X509Certificate2 _certificate;

    [OneTimeSetUp]
    public void OneTimeSetup() {
        _container = MockContainer.Container.Create();
        _certificate = MockContainer.Container.Certificate;
    }

    [SetUp]
    public void SetUp() {
        _client = new Client(
            FitConnectEndpoints.Create(FitConnectEndpoints.EndpointType.Development), clientId,
            clientSecret, _certificate, _certificate, container: _container);
    }

    [Test]
    public void FluentSubscriberPollSubmissions_Success() {
        _client.Subscriber
            .Authenticate(clientId, clientSecret)
            .GetSubmissions("");

        var total = _client.Subscriber.AvailableSubmissions;
        var submissions = _client.Subscriber.Submissions;
        total.Should().Be(3);
        _client.Subscriber.Submissions["80a0aac3-148d-42bb-9366-516ce6355348"].Should().HaveCount(1);
    }

    [Test]
    public void
        FluentSubscriberPollSubmissions_UnauthenticatedPollCall_ThrowsAuthenticationException() {
        Assert.Throws<AuthenticationException>(() =>
            _client.Subscriber
                .GetSubmissions(""));
    }

    [Test]
    public void FluentSubscriberPollSubmissions_AuthenticationFailed() {
        Assert.Throws<InvalidCredentialException>(() => {
            var subscriber = _client.Subscriber
                .Authenticate("WRONG", "WRONG")
                .GetSubmissions("");
        });
    }
}
