using FitConnect.Services.Models;

namespace FitConnect.Services;

public interface ISubmissionService {
    /// <summary>
    ///     <para>@PostMapping("/submissions")</para>
    /// </summary>
    /// <param name="submissionDto">RequestBody</param>
    /// <returns></returns>
    /// <exception cref="NotImplementedException"></exception>
    SubmissionCreatedDto CreateSubmission(CreateSubmissionDto submissionDto);

    /// <summary>
    ///     <para>
    ///         @PutMapping(value = "/submissions/{submissionId}/attachments/{attachmentId}", consumes =
    ///         "application/jose")
    ///     </para>
    /// </summary>
    /// <param name="submissionId">PathVariable</param>
    /// <param name="attachmentId">PathVariable</param>
    /// <param name="encryptedAttachmentContent">RequestBody</param>
    /// <returns></returns>
    /// <exception cref="NotImplementedException"></exception>
    bool AddSubmissionAttachment(string submissionId, string attachmentId,
        string encryptedAttachmentContent);

    /// <summary>
    ///     <para>@PutMapping(value = "/submissions/{submissionId}", consumes = "application/json") </para>
    /// </summary>
    /// <param name="submissionId">PathVariable</param>
    /// <param name="submitSubmission">RequestBody</param>
    /// <returns></returns>
    /// <exception cref="NotImplementedException"></exception>
    SubmissionReducedDto SubmitSubmission(string submissionId,
        SubmitSubmissionDto submitSubmission);

    /// <summary>
    ///     <para>@GetMapping("/submissions")</para>
    /// </summary>
    /// <param name="destinationId">RequestParam</param>
    /// <param name="offset">RequestParam</param>
    /// <param name="limit">RequestParam</param>
    /// <returns></returns>
    /// <exception cref="NotImplementedException"></exception>
    Task<SubmissionsForPickupDto> ListSubmissions(string? destinationId, int offset,
        int limit);

    /// <summary>
    ///     <para>@GetMapping("/submissions/{submissionId}")</para>
    /// </summary>
    /// <param name="submissionId">PathVariable</param>
    /// <returns></returns>
    /// <exception cref="NotImplementedException"></exception>
    SubmissionDto GetSubmission(string submissionId);

    /// <summary>
    ///     <para>
    ///         @GetMapping(value = "/submissions/{submissionId}/attachments/{attachmentId}", produces =
    ///         "application/jose")
    ///     </para>
    /// </summary>
    /// <param name="submissionId">PathVariable</param>
    /// <param name="attachmentId">PathVariable</param>
    /// <returns></returns>
    /// <exception cref="NotImplementedException"></exception>
    string GetAttachment(string submissionId, string attachmentId);
}
